<template>
  <div 
    @mouseenter="handleMouseEnter"
    @mouseleave="handleMouseLeave">
    <MenuReaction 
      v-if="isShowReaction && showEmoji && status !== 'failed'" 
      :id="id" 
      :is-seller="isSeller" 
      @onSuccess="handleReactionSuccess" 
      @onActive="handleOnActive"
      @mouseenter="handleMouseEnterMenu"
      @mouseleave="handleMouseLeaveMenu"
    />
    <ReactionIcon :reaction-list="reactionArr"/>
    <div v-if="reply_from_id && object?.replied_message" class="relative rounded-lg p-2 w-full block bg-scheme-primary bg-opacity-10 mb-2">
        <span class="block">{{ object?.replied_message?.name }}</span>
        <span v-html="helpers.whatsappFormatting(helpers.urlify(helpers.decodeMessage(object?.replied_message?.message)))"></span>
    </div>
    <audio controls>
        <source :src="object?.audio?.audio_url" type="audio/ogg">
        Your browser does not support the audio element.
    </audio>
    <FailedSendMessage v-if="isSeller && status ==='failed'" :message="wa_error_message"/>

    <MenuChatBubble v-if="isShowMenuChat" :is-seller="isSeller" :message="message" :id="id" :chat_type="chatType"/>

    <div v-if="is_forwarded === 1"
          class="absolute  -top-8 gap-1 rounded-tl-xl rounded-tr-xl left-0 block w-full  ">
        <div class="relative flex py-2 items-center">
            <svg xmlns="http://www.w3.org/2000/svg" class="w-5 h-5 fill-current text-gray-400"
                  viewBox="0 0 24 24">
                <path d="M21,12L14,5V9C7,10 4,15 3,20C5.5,16.5 9,14.9 14,14.9V19L21,12Z"/>
            </svg>
            <span class="italic text-gray-400">Forwarded</span>
        </div>
    </div>
    <div :class="`relative flex items-end gap-5 ${assignee ? 'justify-between' : 'justify-end'}` ">
        <div v-if="isFromApplication && isSeller" class="flex items-center gap-2 whitespace-nowrap  rounded-full py-1 px-2 bottom-1 text-xs bg-white text-primary text-right select-none">
          <span class="w-2 h-2 bg-scheme-primary rounded-full"></span>
          <span>{{ applicationAgentName }}</span>
        </div>
        <div v-else-if="assignee"
              class="flex items-center gap-2 whitespace-nowrap  rounded-full py-1 px-2 bottom-1 text-xs bg-white text-primary text-right select-none">
          <span class="w-2 h-2 bg-scheme-primary rounded-full"></span>
          <span>{{ assignee.full_name }}</span>
        </div>
        <div class="flex gap-1 items-center justify-end">
            <span class="block  text-xs text-gray-600 text-right select-none">{{ date }}</span>
            <!-- loading -->
            <template v-if="isSeller && !isLoader">
                <IcMessageStatus :status="status"/>
            </template>
        </div>
    </div>
  <!--  <div-->
  <!--      :class="`relative flex gap-4 w-full ${isLoader ? 'animate-pulse': ''}  ${isSeller ? 'flex-row-reverse' : 'flex-row'}`">-->

  <!--    <div-->
  <!--        :class="`relative overflow-hidden w-14 h-14 object-cover  ${isLoader ? 'bg-gray-400': 'bg-white shadow-md'}  rounded-full flex items-center ${isSeller ? 'order-2 p-3 ' : 'p-3 order-1'}`">-->
  <!--      <div v-if="!isLoader" class="relative block h-14 w-14 overflow-hidden object-cover">-->
  <!--        <ic-person-general />-->
  <!--      </div>-->
  <!--      &lt;!&ndash;        <Avatar :url="'https://source.unsplash.com/random/150x150/?avatar'"/>&ndash;&gt;-->
  <!--    </div>-->
  <!--    <div-->
  <!--        :class="`relative min-w-200 max-w-bubble w-auto mb-12 shadow-sm py-2 pl-2 pr-6 ${isSeller ? 'chat-right order-2 bg-scheme-primary bg-opacity-20' : 'chat-left order-1 bg-gray-100'} ${isLoader ? 'bg-gray-300': ''}`">-->
  <!--      <div class="relative inline-block mb-2 overflow-hidden text-base whitespace-normal pb-2 text-gray-800"-->
  <!--           v-html="message">-->
  <!--      </div>-->


  <!--      <FailedSendMessage v-if="isSeller && isError"/>-->
  <!--    </div>-->
  <!--  </div>-->
  </div>
</template>

<script setup lang="ts">
import FailedSendMessage from "../Utility/FailedSendMessage.vue";
import IcPersonGeneral from "../Icon/IcPersonGeneral.vue";
import IcMessageStatus from "../Icon/IcMessageStatus.vue";
import {onMounted, ref, watch} from "vue";
import MenuChatBubble from "../Menu/MenuChatBubble.vue";
import helpers from "../../utils/helpers";
import ReactionIcon from "../Icon/ReactionIcon.vue";
import ChatRepliedMessage from "./ChatRepliedMessage.vue";
import {useChatListStore} from "../../stores/chat_list";
import {useGeneralStore} from "../../stores/general";
import MenuReaction from "../../components/Menu/MenuReaction.vue";

interface Props {
    isSeller?: boolean
    isError?: boolean
    isLoading?: boolean
    isShowReaction?: boolean
    isShowMenuChat?: boolean
    wa_error_message?: string
    status?: string
    isLoader?: boolean
    id?: bigint
    date?: string
    is_forwarded?: number
    reply_from_id?: string | number
    message?: string
    chatType?: string
    assignee?: {
        full_name?: string
    }
    object?: any
  reactions?: any
  isFromApplication: boolean | null
  applicationAgentName: string | null
}

let props = withDefaults(defineProps<Props>(), {
    isSeller: false,
    isError: false,
    isShowReaction: true,
    isShowMenuChat: true,
    is_forwarded: 0,
    isLoader: false,
    isLoading: false,
    date: null,
    wa_error_message: null,
    reply_from_id: null,
    isFromApplication: null,
  applicationAgentName: null,
    id: null,
    status: null,
    assignee: null,
    chatType: null,
    message: null,
  reactions: [],
    object: {
        audio: {
            audio_url: null,
        }
    },
});


const chat = useChatListStore()
const general = useGeneralStore()
const showEmoji = ref(false)
const emojiActive = ref(false)
const reactionArr = ref([])
const isHoveringMenu = ref(false)
const leaveTimeout = ref()

const handleMouseEnter = () => {
  if (leaveTimeout.value) {
    clearTimeout(leaveTimeout.value);
    leaveTimeout.value = null;
  }
  showEmoji.value = true;
};

const handleMouseEnterMenu = () => {
  isHoveringMenu.value = true;
  handleMouseEnter();
};

const handleMouseLeaveMenu = () => {
  isHoveringMenu.value = false;
  startHideTimeout();
};

const handleMouseLeave = () => {
  startHideTimeout();
};

const startHideTimeout = () => {
  leaveTimeout.value = setTimeout(() => {
    if (!emojiActive.value && !isHoveringMenu.value) {
      showEmoji.value = false;
    }
  }, 200);
};

const handleReactionSuccess = (emojiString: any) => {
  let type_chat = general.chatTab
  let platform = ''

  if(general.chatTab === 'instagram'){
    type_chat = 'instagram'

    if(chat.selectedChatType === 'instagram'){
      platform = 'instagram'
    }
    
    if(chat.selectedChatType === 'facebook'){
      platform = 'page'
    }
  }

  chat.sendEmoji(chat.selectedChat, {
    to: chat?.detail?.customer?.phone,
    chat_id: props.id,
    emoji: emojiString,
    platform: platform,
  },type_chat).then((resp) => {
    if (resp.code === 200) {
      const currentReaction = reactionArr.value.findIndex((rc) => rc.chat_id === props.id && rc.from_chat_user_id === chat.selectedChat)
      if (currentReaction == -1 && reactionArr.value.length == 0) {
        reactionArr.value.push({
          chat_id : props?.id,
          from_chat_user_id: chat.selectedChat,
          value: emojiString
        })
      } else {

        reactionArr.value.splice(currentReaction,1, {
          chat_id : props?.id,
          from_chat_user_id: chat.selectedChat,
          value: emojiString
        })
      }
    } else {
      console.log("400")
    }
  })
}

const handleOnActive= (event) => {
  emojiActive.value = event
}

onMounted(() => {
  reactionArr.value = props.reactions
})

watch(() => props.reactions, (val) => {
  reactionArr.value = val
})


</script>
